@@ -4,6 +4,12 @@ using Gnome;
 using System.Collections;
 using System;
 
+public class ImportException : System.Exception {
+	public ImportException (string msg) : base (msg)
+	{
+	}
+}
+
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
 	bool recurse;
@@ -24,7 +30,13 @@ public class FileImportBackend : ImportBackend {
 
 	private void GetListing (System.IO.DirectoryInfo info)
 	{
-		GetListing (info, info.GetFiles (), recurse);
+		try {
+			GetListing (info, info.GetFiles (), recurse);
+		} catch (System.UnauthorizedAccessException e) {
+			System.Console.WriteLine ("Unable to access directory {0}", info.FullName);
+		} catch (System.Exception e) {
+			System.Console.WriteLine ("{0}", e.ToString ());
+		}
 	}
 
 	private void GetListing (System.IO.DirectoryInfo info, System.IO.FileInfo [] files, bool recurse)
@@ -61,7 +73,7 @@ public class FileImportBackend : ImportBackend {
 	public override int Prepare ()
 	{
 		if (file_paths != null)
-			throw new Exception ("Busy");
+			throw new ImportException ("Busy");
 
 		file_paths = new ArrayList ();
 
@@ -103,7 +115,7 @@ public class FileImportBackend : ImportBackend {
 	{
 		string name = System.IO.Path.GetFileName (path);
 		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		DateTime time = img.Date ();
+		DateTime time = img.Date;
 		
 		string dest_dir = String.Format ("{0}{1}{2}{1}{3}{1}{4}",
 						 FSpot.Global.PhotoDirectory,
@@ -123,31 +135,37 @@ public class FileImportBackend : ImportBackend {
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
 		if (file_paths == null)
-			throw new Exception ("Prepare() was not called");
+			throw new ImportException ("Prepare() was not called");
 
 		if (this.count == file_paths.Count)
-			throw new Exception ("Already finished");
+			throw new ImportException ("Already finished");
 
 		// FIXME Need to get the EXIF info etc.
 		string path = (string) file_paths [this.count];
-
-		if (copy) {
-			string dest = ChooseLocation (path);
-
-			System.IO.File.Copy (path, dest);
-			path = dest;
-		}
 		
-		photo = store.Create (path, out thumbnail);
-
-		if (tags != null) {
-			foreach (Tag t in tags) {
-				photo.AddTag (t);
+		try {
+			if (copy) {
+				string dest = ChooseLocation (path);
+				System.IO.File.Copy (path, dest);
+				photo = store.Create (dest, path, out thumbnail);
+				path = dest;
+			} else {
+                photo = store.Create (path, out thumbnail);
 			}
-			store.Commit(photo);
+			
+			if (tags != null) {
+				foreach (Tag t in tags) {
+					photo.AddTag (t);
+				}
+				store.Commit(photo);
+			}
+			imported_photos.Add (photo);
+			
+		} catch (System.Exception e) {
+			System.Console.WriteLine ("Error importing {0}\n{1}", path, e.ToString ());
+			thumbnail = null;
+			photo = null;
 		}
-		
-		imported_photos.Add (photo);
 
 		this.count ++;
 		count = this.count;
@@ -158,7 +176,7 @@ public class FileImportBackend : ImportBackend {
 	public override void Cancel ()
 	{
 		if (imported_photos == null)
-			throw new Exception ("Not doing anything");
+			throw new ImportException ("Not doing anything");
 
 		foreach (Photo p in imported_photos) {
 			if (copy) {
@@ -178,7 +196,7 @@ public class FileImportBackend : ImportBackend {
 	public override void Finish ()
 	{
 		if (file_paths == null)
-			throw new Exception ("Not doing anything");
+			throw new ImportException ("Not doing anything");
 
 		file_paths = null;
 
