@@ -31,7 +31,6 @@ namespace FSpot {
 	public class FolderExport : GladeDialog {
 		IBrowsableCollection selection;
 		[Glade.Widget] Gtk.ScrolledWindow thumb_scrolledwindow;
-		[Glade.Widget] Gtk.Entry uri_entry;
 		[Glade.Widget] Gtk.Entry name_entry;
 		[Glade.Widget] Gtk.Entry description_entry;
 
@@ -44,8 +43,11 @@ namespace FSpot {
 		[Glade.Widget] Gtk.RadioButton plain_radio;
 
 		[Glade.Widget] Gtk.SpinButton size_spin;
+		
+		[Glade.Widget] Gtk.HBox chooser_hbox;
 
 		Gnome.Vfs.Uri dest;
+		Gtk.FileChooserButton uri_chooser;
 		
 		int photo_index;
 		bool open;
@@ -95,9 +97,16 @@ namespace FSpot {
 			thumb_scrolledwindow.Add (view);
 			HandleSizeActive (null, null);
 			name_entry.Text = gallery_name;
-			uri_entry.Text = System.IO.Path.Combine (FSpot.Global.HomeDirectory, "Desktop");
-			if (!System.IO.Directory.Exists (uri_entry.Text))
-				uri_entry.Text = FSpot.Global.HomeDirectory;
+
+			string uri_path = System.IO.Path.Combine (FSpot.Global.HomeDirectory, "Desktop");
+			if (!System.IO.Directory.Exists (uri_path))
+			        uri_path = FSpot.Global.HomeDirectory;
+
+			uri_chooser = new Gtk.FileChooserButton (Mono.Posix.Catalog.GetString ("Select Export Folder"),
+								 Gtk.FileChooserAction.SelectFolder);
+			uri_chooser.SetFilename (uri_path);
+			uri_chooser.LocalOnly = false;
+			chooser_hbox.PackStart (uri_chooser);
 
 			Dialog.ShowAll ();
 
@@ -174,9 +183,7 @@ namespace FSpot {
 					progress_dialog.Message = Mono.Posix.Catalog.GetString ("Error While Transferring");
 				}
 
-				if (open)
-					GnomeUtil.UrlShow (null, target.ToString ());
-
+				GnomeUtil.UrlShow (null, target.ToString ());
 			} catch (System.Exception e) {
 				progress_dialog.Message = e.ToString ();
 				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error Transferring");
@@ -188,7 +195,7 @@ namespace FSpot {
 				Dialog.Destroy ();
 			}
 		}
-		
+
 		private int Progress (Gnome.Vfs.XferProgressInfo info)
 		{
 			progress_dialog.ProgressText = info.Phase.ToString ();
@@ -235,8 +242,8 @@ namespace FSpot {
 				Dialog.Destroy ();
 				return;
 			}
-
-			dest = new Gnome.Vfs.Uri (uri_entry.Text);
+			
+			dest = new Gnome.Vfs.Uri (uri_chooser.Uri);
 			open = open_check.Active;
 			scale = scale_check.Active;
 			gallery_name = name_entry.Text;
@@ -273,7 +280,7 @@ namespace FSpot {
 		protected ScaleRequest [] requests;
 		
 		protected string [] pixbuf_keys = {"quality"};
-		protected string [] pixbuf_values = {"75"};
+		protected string [] pixbuf_values = {"95"};
 
 		protected struct ScaleRequest {
 			public string Name;
@@ -483,8 +490,8 @@ namespace FSpot {
 
 		private void CreateComments(string photo_path, int photo_index)
 		{
-			StreamWriter comment = File.CreateText(SubdirPath  ("comments", AlternateName (photo_index, "txt")));
-			comment.Write("<span>photo " + photo_index + "</span> ");
+			StreamWriter comment = File.CreateText(SubdirPath  ("comments", photo_index + 1 + ".txt"));
+			comment.Write("<span>photo " + (photo_index + 1) + "</span> ");
 			comment.Write (collection [photo_index].Description + "\n");
 			comment.Close();
 		}
