@@ -1,4 +1,5 @@
 using System;
+using Mono.Unix;
 using Gtk;
 using Gdk;
 using GLib;
@@ -28,7 +29,6 @@ namespace FSpot {
 		public Gdk.Rectangle legend;
 		public Gdk.Rectangle action;
 
-
 		Pango.Layout [] tick_layouts;
 		int [] box_counts = new int [0];
 		int box_count_max;
@@ -50,6 +50,17 @@ namespace FSpot {
 				    min_limit.SetPosition (0, false);
 				    max_limit.SetPosition (adaptor.Count () - 1, false);
 				}
+			
+				if (adaptor is TimeAdaptor) {
+					MainWindow.SetTip (left, Catalog.GetString ("More dates"));
+					MainWindow.SetTip (right, Catalog.GetString ("More dates"));
+				} else if (adaptor is DirectoryAdaptor) {
+					MainWindow.SetTip (left, Catalog.GetString ("More directories"));
+					MainWindow.SetTip (right, Catalog.GetString ("More directories"));
+				} else {
+					MainWindow.SetTip (left, Catalog.GetString ("More"));
+					MainWindow.SetTip (right, Catalog.GetString ("More"));
+				}
 				
 				adaptor.Changed += HandleAdaptorChanged;
 			}
@@ -58,6 +69,12 @@ namespace FSpot {
 			}
 		}
 
+		public int GlassPosition {
+			get {
+				return glass.Position;
+			}
+		}
+
 		private void HandleAdaptorChanged (GroupAdaptor adaptor)
 		{
 			bool size_changed = box_counts.Length != adaptor.Count ();
@@ -263,6 +280,9 @@ namespace FSpot {
 
 		protected override bool OnButtonPressEvent (Gdk.EventButton args)
 		{
+			if (args.Button == 3)
+				return DrawOrderMenu (args); 
+
 			double x = args.X + action.X;
 			double y = args.Y + action.Y;
 
@@ -349,8 +369,10 @@ namespace FSpot {
 			attr.Wclass = WindowClass.InputOnly;
 			attr.EventMask = (int) Events;
 			attr.EventMask |= (int) (EventMask.ButtonPressMask | 
-				EventMask.ButtonReleaseMask | 
-				EventMask.PointerMotionMask);
+						 EventMask.KeyPressMask |
+						 EventMask.KeyReleaseMask |
+						 EventMask.ButtonReleaseMask | 
+						 EventMask.PointerMotionMask);
 				
 			event_window = new Gdk.Window (GdkWindow, attr, (int) (WindowAttributesType.X | WindowAttributesType.Y));
 			event_window.UserData = this.Handle;
@@ -393,13 +415,16 @@ namespace FSpot {
 				bounds.Y = selector.background.Y;
 				bounds.X = selector.BoxX (item);
 				bounds.Width = Math.Max (selector.BoxX (item + 1) - bounds.X, 1);
-				
+
+				if (item < 0 || item > selector.box_counts.Length - 1)
+					return;
+
 				double percent = selector.box_counts [item] / (double) Math.Max (selector.box_count_max, 1);
 				bar = bounds;
 				bar.Height = (int) Math.Ceiling ((bounds.Height - selector.box_top_padding) * percent);
 				bar.Y += bounds.Height - bar.Height - 1;
 				
-				bar.Inflate (-selector.box_spacing, 0);
+				bar.Inflate (- selector.box_spacing, 0);
 			}
 			
 			public Gdk.Rectangle Bounds {
@@ -474,6 +499,35 @@ namespace FSpot {
 				GdkWindow.DrawRectangle (Style.ForegroundGC (State), true, area);
 			}
 		}
+		
+		protected override bool OnPopupMenu ()
+		{
+			DrawOrderMenu (null);
+			return true;
+		}
+
+		private bool DrawOrderMenu (Gdk.EventButton args)
+		{
+			Gtk.Menu order_menu = new Gtk.Menu();
+			
+			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Month"),
+					      MainWindow.Toplevel.HandleArrangeByTime, true, (adaptor is TimeAdaptor), true);
+			
+			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Folder"),
+					      MainWindow.Toplevel.HandleArrangeByDirectory, true, (adaptor is DirectoryAdaptor), true);
+
+			GtkUtil.MakeMenuSeparator (order_menu);
+
+			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("_Reverse Order"),
+					      MainWindow.Toplevel.HandleReverseOrder, true, adaptor.OrderAscending, false);
+
+			if (args != null)
+				order_menu.Popup (null, null, null, args.Button, args.Time);
+			else
+				order_menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
+			
+			return true;
+		}
 
 		public abstract class Manipulator {
 			protected GroupSelector selector;
@@ -526,6 +580,14 @@ namespace FSpot {
 
 				return true;
 			}
+			
+			protected bool PositionValid (int position)
+			{
+				if (position < 0 || position > selector.box_counts.Length - 1)
+					return false;
+
+				return true;
+			}
 
 			public virtual void UpdateDrag (double x, double y)
 			{
@@ -538,8 +600,7 @@ namespace FSpot {
 						return;
 				}
 
-				System.Console.WriteLine ("scroll {0}, x {1}", scroll, x);
-
+				//System.Console.WriteLine ("scroll {0}, x {1}", scroll, x);
 				DragOffset = (int)x - DragStart.X;
 			}
 
@@ -581,6 +642,9 @@ namespace FSpot {
 
 			public void SetPosition (int position, bool update)
 			{
+				if (! PositionValid (position))
+					return;
+				    
 				Rectangle then = Bounds ();
 				this.position = position;
 				Rectangle now = Bounds ();
@@ -709,40 +773,44 @@ namespace FSpot {
 
 			public override void Draw (Rectangle area)
 			{
+				if (! PositionValid (Position))
+					return;
+
 				Rectangle inner = InnerBounds ();
 				Rectangle bounds = Bounds ();
 				
-				if (bounds.Intersect (area, out area)) {
-					int i = 0;
-
-					Rectangle box = inner;
-					box.Width -= 1;
-					box.Height -= 1;
-					while (i < border) {
-						box.Inflate (1, 1);
-						
-						selector.Style.BackgroundGC (State).ClipRectangle = area;
-						selector.GdkWindow.DrawRectangle (selector.Style.BackgroundGC (State), 
-										  false, box);
-						i++;
-					}
-
-
-					Style.PaintFlatBox (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
-							    bounds.Width, handle_height);
-
-					Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
-							    bounds.Width, handle_height, Orientation.Horizontal);
+				if (! bounds.Intersect (area, out area))
+				    return;
+				    
+				int i = 0;
+				
+				Rectangle box = inner;
+				box.Width -= 1;
+				box.Height -= 1;
+				while (i < border) {
+					box.Inflate (1, 1);
 					
-					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
-							   area, selector, "glass", bounds.X, bounds.Y, bounds.Width, bounds.Height);
-
-					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							   area, selector, "glass", inner.X, inner.Y, inner.Width, inner.Height);
-
+					selector.Style.BackgroundGC (State).ClipRectangle = area;
+					selector.GdkWindow.DrawRectangle (selector.Style.BackgroundGC (State), 
+									  false, box);
+					i++;
 				}
+				
+				
+				Style.PaintFlatBox (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
+						    bounds.Width, handle_height);
+				
+				Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						   area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
+						   bounds.Width, handle_height, Orientation.Horizontal);
+				
+				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
+						   area, selector, "glass", bounds.X, bounds.Y, bounds.Width, bounds.Height);
+				
+				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						   area, selector, "glass", inner.X, inner.Y, inner.Width, inner.Height);
+				
 			}
 			
 			public override void PositionChanged ()
@@ -1016,11 +1084,13 @@ namespace FSpot {
 			right.ButtonReleaseEvent += HandleScrollReleaseEvent;
 			right_delay = new Delay (50, new GLib.IdleHandler (HandleScrollRight));
 			//right.Clicked += HandleScrollRight;
-
+            
 			this.Put (left, 0, 0);
 			this.Put (right, 100, 0);
 			left.Show ();
 			right.Show ();
+
+			CanFocus = true;
 			
 			Mode = RangeType.Min;
 			UpdateButtons ();
