@@ -2,6 +2,25 @@ using System;
 using SemWeb;
 
 namespace FSpot {
+	public class InfoDialog : Gtk.Dialog {
+		InfoDisplay info_display;
+		public InfoDisplay InfoDisplay {
+			get { return info_display; }
+		}
+
+		public InfoDialog (Gtk.Window parent) : base (Mono.Posix.Catalog.GetString ("Metadata Browser"),
+											    parent,
+											    Gtk.DialogFlags.NoSeparator | Gtk.DialogFlags.DestroyWithParent)
+		{
+			info_display = new InfoDisplay ();
+			SetDefaultSize (400, 400);
+			Gtk.ScrolledWindow scrolled = new Gtk.ScrolledWindow ();
+			VBox.PackStart (scrolled);
+			scrolled.Add (info_display);
+		}
+	}
+
+
 	public class InfoDisplay : Gtk.HTML {
 		public InfoDisplay () 
 		{
@@ -133,6 +152,16 @@ namespace FSpot {
 				} 
 				
 				if (store.StatementCount > 0) {
+#if false
+					using (System.IO.Stream xmpstream = System.IO.File.OpenWrite ("tmp.xmp")) {
+						xmpstream.Length = 0;
+						FSpot.Xmp.XmpFile file;
+
+						file = new FSpot.Xmp.XmpFile ();
+						store.Select (file);
+						file.Save (xmpstream);
+					}
+#endif
 					empty = false;
 					stream.Write ("<tr><th align=left bgcolor=\"" + ig + "\" colspan=2>" 
 						      + Mono.Posix.Catalog.GetString ("Extended Metadata") + "</th><tr>");
@@ -174,9 +203,9 @@ namespace FSpot {
 					// handle multiple items however it wants.
 					msg = String.Format ("<tr><td valign=top align=center bgcolor=\"{0}\">" 
 							     + "<b>{1}</b></td></tr>", ig,
-							     Mono.Posix.Catalog.GetString ("No active image"));
+							     Mono.Posix.Catalog.GetString ("No active photo"));
 				} else if (missing) {
-					string text = String.Format (Mono.Posix.Catalog.GetString ("The image \"{0}\" does not exist"), photo.DefaultVersionUri);
+					string text = String.Format (Mono.Posix.Catalog.GetString ("The photo \"{0}\" does not exist"), photo.DefaultVersionUri);
 					msg = String.Format ("<tr><td valign=top align=center bgcolor=\"{0}\">" 
 							     + "<b>{1}</b></td></tr>", ig, text);
 				} else {
@@ -208,7 +237,6 @@ namespace FSpot {
 					string predicate = sub.Predicate.ToString ();
 					string title = System.IO.Path.GetFileName (predicate);
 					string value = ((Literal)(sub.Object)).Value;
-					string prefix;
 					string vc = "";
 					
 					Description.GetDescription (substore, sub, out title, out value);
