@@ -103,6 +103,21 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return string.Compare (photo1.name, photo2.name);
 	}
 
+	public class CompareDateName : IComparer {
+		public int Compare (object obj1, object obj2) {
+			Photo p1 = (Photo)obj1;
+			Photo p2 = (Photo)obj2;
+
+			int result = Photo.CompareDate (p1, p2);
+			
+			if (result == 0)
+				result = CompareName (p1, p2);
+
+			return result;
+		}
+
+	}
+
 	public class CompareDirectory : IComparer {
 		public int Compare (object obj1, object obj2) {
 			Photo p1 = (Photo)obj1;
@@ -117,6 +132,14 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	public class RandomSort : IComparer {
+		Random random = new Random ();
+		
+		public int Compare (object obj1, object obj2) {
+			return random.Next (-5, 5);
+		}
+	}
+
 	// The time is always in UTC.
 	private DateTime time;
 	public DateTime Time {
@@ -475,7 +498,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
 			jimg.SetDescription (this.Description);
-			jimg.SetDateTimeOriginal (this.Time);
+			jimg.SetDateTimeOriginal (this.Time.ToLocalTime ());
 			jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
 
 			jimg.SaveMetaData (path);
@@ -680,7 +703,7 @@ public class PhotoStore : DbStore {
 	{
 		FSpot.ImageFile img = FSpot.ImageFile.Create (origPath);
 		long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
-		string description = img.Description != null ? img.Description : "";
+		string description = img.Description != null  ? img.Description.Split ('\0') [0] : "";
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
