@@ -4,6 +4,29 @@ using Mono.Posix;
 
 namespace FSpot
 {
+	public class PreferenceSetting
+	{
+		private string key;
+
+		public string Key {
+			get { return key; }
+		}
+		
+		public object Value {
+			get {
+				return Preferences.Get (key);
+			}
+			set {
+				Preferences.Set (key, Value);
+			}
+		}
+		
+		public PreferenceSetting (string key)
+		{
+			this.key = key;
+		}
+	}
+
 	public class Preferences
 	{
 		public const string MAIN_WINDOW_MAXIMIZED = "/apps/f-spot/ui/maximized";
@@ -25,9 +48,12 @@ namespace FSpot
 		public const string SHOW_DATES = "/apps/f-spot/ui/show_dates";
 		public const string EXPANDED_TAGS = "/apps/f-spot/ui/expanded_tags";
 		
+		public const string GLASS_POSITION = "/apps/f-spot/ui/glass_position";
+		public const string GROUP_ADAPTOR = "/apps/f-spot/ui/group_adaptor";
+		public const string GROUP_ADAPTOR_ORDER_ASC = "/apps/f-spot/ui/group_adaptor_sort_asc";
+		
 		public const string SIDEBAR_POSITION = "/apps/f-spot/ui/sidebar_size";
 		public const string ZOOM = "/apps/f-spot/ui/zoom";
-		public const string ICON_VIEW_POSITION = "/apps/f-spot/ui/icon_view_position";
 
 		public const string EXPORT_FLICKR_SCALE = "/apps/f-spot/export/flickr/scale";
 		public const string EXPORT_FLICKR_SIZE = "/apps/f-spot/export/flickr/size";
@@ -45,8 +71,12 @@ namespace FSpot
 		public const string EXPORT_GALLERY_BROWSER = "/apps/f-spot/export/gallery/browser";
 		public const string EXPORT_GALLERY_META = "/apps/f-spot/export/gallery/meta";
 
+		public const string SCREENSAVER_TAG = "/apps/f-spot/screensaver/tag_id";
+
 		public const string METADATA_EMBED_IN_IMAGE = "/apps/f-spot/metadata/embed_in_image";
 
+		public const string GNOME_SCREENSAVER_THEME = "/apps/gnome-screensaver/themes";
+
 		static GConf.Client client;
 		static GConf.NotifyEventHandler changed_handler;
 
@@ -58,6 +88,7 @@ namespace FSpot
 
 					changed_handler = new GConf.NotifyEventHandler (OnSettingChanged);
 					client.AddNotify ("/apps/f-spot", changed_handler);
+					client.AddNotify ("/apps/gnome-screensaver/themes", changed_handler);
 				}
 				return client;
 			}
@@ -77,6 +108,11 @@ namespace FSpot
 			case MAIN_WINDOW_MAXIMIZED:
 				return false;
 
+			case GROUP_ADAPTOR:
+			case GLASS_POSITION:
+			case GROUP_ADAPTOR_ORDER_ASC:
+				return null;
+
 			case SHOW_TOOLBAR:
 			case SHOW_SIDEBAR:
 			case SHOW_TIMELINE:
@@ -86,8 +122,9 @@ namespace FSpot
 		
 			case SIDEBAR_POSITION:
 			case ZOOM:
-			case ICON_VIEW_POSITION:
 				return null;
+			case SCREENSAVER_TAG:
+				return 1;
 			}
 
 			return null;
