@@ -1,6 +1,7 @@
 using Gdk;
 using Gnome;
 using Gtk;
+using Mono.Posix;
 using Mono.Data.SqliteClient;
 using System.Collections;
 using System.IO;
@@ -135,6 +136,16 @@ public class Tag : DbItem, IComparable {
 			return Category.CompareTo (tag.Category);
 		}
 	}
+	
+	public bool IsAncestorOf (Tag tag)
+	{
+		for (Category parent = tag.Category; parent != null; parent = parent.Category) {
+			if (parent == this)
+				return true;
+		}
+
+		return false;
+	}
 }
 
 
@@ -154,6 +165,22 @@ public class Category : Tag {
 		}
 	}
 
+	// Appends all of this categories descendents to the list
+	public void AddDescendentsTo (ArrayList list)
+	{
+		foreach (Tag tag in children) {
+			if (! list.Contains (tag))
+				list.Add (tag);
+
+			if (! (tag is Category))
+				continue;
+
+			Category cat = tag as Category;
+
+			cat.AddDescendentsTo (list);
+		}
+	}
+
 	public void AddChild (Tag child)
 	{
 		children.Add (child);
@@ -189,6 +216,8 @@ public class InvalidTagOperationException : InvalidOperationException {
 
 }
 
+// Sorts tags into an order that it will be safe to delete
+// them in (eg children first).
 public class TagRemoveComparer : IComparer {
 	public int Compare (object obj1, object obj2) 
 	{
@@ -200,37 +229,16 @@ public class TagRemoveComparer : IComparer {
        
 	public int Compare (Tag t1, Tag t2)
 	{
-		bool c1 = t1 is Category;
-		bool c2 = t2 is Category;
-		
-		if (c1 == c2) {
-			if (c1 && c2) {
-				for (Category cat = t1 as Category; cat.Category != null; cat = cat.Category) {
-					if (cat == t2.Category)
-						return 1;
-				}
-			}
-			
-			return t1.Name.CompareTo (t2.Name);
-		}
-
-		if (c1)
+		if (t1.IsAncestorOf (t2))
 			return 1;
-		else
+		else if (t2.IsAncestorOf (t1))
 			return -1;
+		else
+			return 0;
 	}
 }
 
-public delegate void TagCreatedHandler(Tag t);
-public delegate void TagChangedHandler(Tag t);
-public delegate void TagDeletedHandler(Tag t);
-
 public class TagStore : DbStore {
-
-	public event TagCreatedHandler TagCreated;
-	public event TagChangedHandler TagChanged;
-	public event TagDeletedHandler TagDeleted;
-	
 	Category root_category;
 	public Category RootCategory {
 		get {
@@ -257,6 +265,38 @@ public class TagStore : DbStore {
 		}
 	}
 
+	public Tag GetTagByName (string name)
+	{
+		foreach (Tag t in this.item_cache.Values)
+			if (t.Name.ToLower () == name.ToLower ())
+				return t;
+
+		return null;
+	}
+
+	public Tag GetTagById (int id)
+	{
+		foreach (Tag t in this.item_cache.Values)
+			if (t.Id == id)
+				return t;
+
+		return null;
+	}
+
+	public Tag [] GetTagsByNameStart (string s)
+	{
+		ArrayList l = new ArrayList ();
+		foreach (Tag t in this.item_cache.Values) {
+			if (t.Name.ToLower ().StartsWith (s.ToLower ()))
+				l.Add (t);
+		}
+
+		if (l.Count == 0)
+			return null;
+
+		return (Tag []) (l.ToArray (typeof (Tag)));
+	}
+
 	// In this store we keep all the items (i.e. the tags) in memory at all times.  This is
 	// mostly to simplify handling of the parent relationship between tags, but it also makes it
 	// a little bit faster.  We achieve this by passing "true" as the cache_is_immortal to our
@@ -287,9 +327,6 @@ public class TagStore : DbStore {
 
 			tag.SortPriority = Convert.ToInt32 (reader[3]);
 			AddToCache (tag);
-			
-			if (tag.Name == "Hidden")
-				hidden = tag;
 		}
 
 		reader.Close ();
@@ -318,6 +355,9 @@ public class TagStore : DbStore {
 		}
 		reader.Close ();
 		command.Dispose ();
+
+		if (FSpot.Core.Database.Meta.HiddenTagId.Value != null)
+			hidden = LookupInCache ((uint) FSpot.Core.Database.Meta.HiddenTagId.ValueAsInt) as Tag;
 	}
 
 
@@ -343,36 +383,33 @@ public class TagStore : DbStore {
 
 	private void CreateDefaultTags ()
 	{
-		Tag favorites_tag = CreateTag (RootCategory, "Favorites");
+		Tag favorites_tag = CreateTag (RootCategory, Catalog.GetString ("Favorites"));
 		favorites_tag.StockIconName = "f-spot-favorite.png";
 		favorites_tag.SortPriority = -10;
 		Commit (favorites_tag);
 
-		Tag hidden_tag = CreateTag (RootCategory, "Hidden");
+		Tag hidden_tag = CreateTag (RootCategory, Catalog.GetString ("Hidden"));
 		hidden_tag.StockIconName = "f-spot-hidden.png";
 		hidden_tag.SortPriority = -9;
 		this.hidden = hidden_tag;
 		Commit (hidden_tag);
+		FSpot.Core.Database.Meta.HiddenTagId.ValueAsInt = (int) hidden_tag.Id;
+		FSpot.Core.Database.Meta.Commit (FSpot.Core.Database.Meta.HiddenTagId);
 
-		Tag people_category = CreateCategory (RootCategory, "People");
+		Tag people_category = CreateCategory (RootCategory, Catalog.GetString ("People"));
 		people_category.StockIconName = "f-spot-people.png";
 		people_category.SortPriority = -8;
 		Commit (people_category);
 
-		Tag places_category = CreateCategory (RootCategory, "Places");
+		Tag places_category = CreateCategory (RootCategory, Catalog.GetString ("Places"));
 		places_category.StockIconName = "f-spot-places.png";
 		places_category.SortPriority = -8;
 		Commit (places_category);
 
-		Tag events_category = CreateCategory (RootCategory, "Events");
+		Tag events_category = CreateCategory (RootCategory, Catalog.GetString ("Events"));
 		events_category.StockIconName = "f-spot-events.png";
 		events_category.SortPriority = -7;
 		Commit (events_category);
-
-		Tag other_category = CreateCategory (RootCategory, "Other");
-		other_category.StockIconName = "f-spot-other.png";
-		other_category.SortPriority = -6;
-		Commit (other_category);
 	}
 
 
@@ -381,7 +418,8 @@ public class TagStore : DbStore {
 	public TagStore (SqliteConnection connection, bool is_new)
 		: base (connection, true)
 	{
-		root_category = new Category (null, 0, "root");
+		// The label for the root category is used in new and edit tag dialogs
+		root_category = new Category (null, 0, Mono.Posix.Catalog.GetString ("(None)"));
 
 		if (! is_new) {
 			LoadAllTags ();
@@ -419,9 +457,10 @@ public class TagStore : DbStore {
 		uint id = InsertTagIntoTable (category, name, false);
 
 		Tag tag = new Tag (category, id, name);
+
 		AddToCache (tag);
-		if (TagCreated != null)
-			TagCreated(tag);	
+		EmitAdded (tag);
+		
 		return tag;
 	}
 
@@ -433,9 +472,10 @@ public class TagStore : DbStore {
 		uint id = InsertTagIntoTable (parent_category, name, true);
 
 		Category new_category = new Category (parent_category, id, name);
+
 		AddToCache (new_category);
-		if (TagCreated != null)
-			TagCreated(new_category);
+		EmitAdded (new_category);
+
 		return new_category;
 	}
 
@@ -466,7 +506,7 @@ public class TagStore : DbStore {
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
-		TagDeleted((Tag)item);
+		EmitRemoved (item);
 	}
 
 
@@ -505,8 +545,7 @@ public class TagStore : DbStore {
 
 		command.Dispose ();
 		
-		if (TagChanged != null)
-			TagChanged(tag);
+		EmitChanged (tag);
 	}
 
 
